==> transmedika_one_to_one_call-1.0.6.tar.gz: lib/webrtc/one_to_one/bloc/one_to_one_bloc.dart <==
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/consultation.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/registration_session.dart';

import '../../../models/response/socket_response.dart';
import '../../../websocket/state/socket_state.dart';
import '../../constants.dart';
import '../usecase/one_to_one_usecase_service.dart';
import 'one_to_one_event.dart';
import 'one_to_one_state.dart';

class OneToOneBloc extends Bloc<OneToOneUiUIEvent, OneToOneUiState>{
  OneToOneBloc(
      {required OneToOneUseCaseService oneToOneUseCaseService}):
        _oneToOneUseCaseService = oneToOneUseCaseService, super(const OneToOneUiState(
        socketState: SocketState(socketConnectionState: SocketConnectionState.idle)
  )) {
    on<OnConnectToSocket>(
        _onConnectToSocket
    );

    on<OnSocketOpen>(
        _onSocketOpen
    );

    on<OnSocketMessage>(
        _onSocketMessage
    );

    on<OnSocketClose>(
        _onSocketClose
    );

    on<OnRegister>(
        _onRegisterToSocket
    );

    on<OnLocalStream>(
        _onLocalStream
    );

    on<OnAddRemoteStream>(
        _onAddRemoteStream
    );

    on<OnRemoveRemoteStream>(
        _onRemoveRemoteStream
    );

    on<OnCloseSession>(
      _onCloseSession
    );

    on<OnCallAccepted>(
      _onCallAccepted
    );

    on<OnCallRejected>(
      _onCallRejected
    );

    on<OnStopCall>(
      _onStopCall
    );

    on<OnMute>(
      _onMute
    );

    on<OnSwitchCamera>(
        _onSwitchCamera
    );

    on<OnEnableVideo>(
        _onEnableVideo
    );
  }


  final OneToOneUseCaseService _oneToOneUseCaseService;

  _onCloseSession(OnCloseSession event, Emitter<OneToOneUiState> emit) async{
    await _oneToOneUseCaseService.closeSession();
  }

  _onRegisterToSocket(OnRegister event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(callState: CallState.registering));
    await _oneToOneUseCaseService.register(from: state.from!, to: state.to!, isCalling: state.isCalling!);
  }

  _onSocketOpen(OnSocketOpen event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(
        socketState: const SocketState(
            socketConnectionState: SocketConnectionState.open
        )
    ));
    add(OnRegister());
  }

  _onLocalStream(OnLocalStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(localStream: event.mediaStream));
  }

  _onAddRemoteStream(OnAddRemoteStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(remoteStream: event.mediaStream));
  }

  _onRemoveRemoteStream(OnRemoveRemoteStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(removeRemoteStream: event.mediaStream));
  }

  _onCallRejected(OnCallRejected event, Emitter<OneToOneUiState> emit) async {
    _oneToOneUseCaseService.reject(from: state.registrationSessionFrom!, to: state.registrationSessionTo!);
    emit(state.copyWith(callState: CallState.callRejectedByUser));
    await _oneToOneUseCaseService.close();
  }

  _onMute(OnMute event, Emitter<OneToOneUiState> emit) async {
    if(state.mute == true){
      emit(state.copyWith(mute: false));
    }else{
      emit(state.copyWith(mute: true));
    }
    _oneToOneUseCaseService.muteMic();
  }

  _onEnableVideo(OnEnableVideo event, Emitter<OneToOneUiState> emit) async {
    if(state.enableVideo == true){
      emit(state.copyWith(enableVideo: false));
    }else{
      emit(state.copyWith(enableVideo: true));
    }
    _oneToOneUseCaseService.enableVideo();
  }

  _onSwitchCamera(OnSwitchCamera event, Emitter<OneToOneUiState> emit) async {
    if(state.frontCamera == true){
      emit(state.copyWith(frontCamera: false));
    }else{
      emit(state.copyWith(frontCamera: true));
    }
    _oneToOneUseCaseService.switchCamera();
  }

  _onStopCall(OnStopCall event, Emitter<OneToOneUiState> emit) async {
    await _oneToOneUseCaseService.closeSession();
    await _oneToOneUseCaseService.close();
  }

  _onCallAccepted(OnCallAccepted event, Emitter<OneToOneUiState> emit) async {
    emit(state.copyWith(callState: CallState.callAcceptedByUser));
    await _createLocalPeerConnection();
    _createOffer(
        state.registrationSessionFrom!,
        state.registrationSessionTo!,
        state.isCalling!,
        Consultation(
            consultationId: state.consultationId!,
            callerUUID: state.mySelf!.uuid,
            calleeUUID:  state.other!.uuid
        )
    );
  }

  _onSocketMessage(OnSocketMessage event, Emitter<OneToOneUiState> emit) async{
    Map<String, dynamic> valueMap = json.decode(event.message);
    SocketResponse response = SocketResponse.fromJson(valueMap);

    emit(state.copyWith(
        socketState: state.socketState?.copyWith(
          message: event.message,
          messageModel: response
        )
    ));

    switch (response.id) {
      case Constants.registerResponse:
        if (response.response == Constants.accepted) {
          emit(state.copyWith(
            callState: CallState.registerAccepted,
            registrationSessionFrom: response.registrationSession
          ));
          _oneToOneUseCaseService.notifyCaller(from: response.registrationSession!, to: state.to!);
        } else {
          emit(state.copyWith(
            callState: CallState.registerRejected,
            registrationSessionFrom: response.registrationSession
          ));
        }
        break;
      case Constants.notifyCallerActive:
        emit(state.copyWith(
            callState: CallState.notifyMe,
            registrationSessionTo: response.from
        ));

        if(state.isCalling == true){
          await _createLocalPeerConnection();
          _createOffer(
            state.registrationSessionFrom!,
            state.registrationSessionTo!,
            state.isCalling!,
            Consultation(
                consultationId: state.consultationId!,
                callerUUID: state.mySelf!.uuid,
                calleeUUID:  state.other!.uuid
            )
          );
        }
        break;
      case Constants.notifyCallerNotActive:
        emit(state.copyWith(
            callState: CallState.missedCall,
            registrationSessionFrom: response.from
        ));
        break;
      case Constants.callResponse:
        if (response.response == Constants.accepted) {
          emit(state.copyWith(callState: CallState.callAccepted));
          _oneToOneUseCaseService.receiveVideoAnswer(response);
        } else {
          emit(state.copyWith(callState: CallState.callRejected));
        }
        break;
      case Constants.incomingCall:
        emit(state.copyWith(
          callState: CallState.incomingCall,
          registrationSessionTo: response.from!
        ));
        break;
      case Constants.startCommunication:
        emit(state.copyWith(callState: CallState.startCommunication));
        _oneToOneUseCaseService.receiveVideoAnswer(response);
        break;
      case Constants.iceCandidate:
        emit(state.copyWith(callState: CallState.iceCandidate));
        _oneToOneUseCaseService.iceCandidate(response);
        break;
      case Constants.stopCommunication:
        emit(state.copyWith(callState: CallState.stopCommunication));
        break;
      default:
        emit(state.copyWith(callState: CallState.unknown));
        debugPrint(Constants.unknown);
    }
  }

  _createLocalPeerConnection() async{
    await _oneToOneUseCaseService.createLocalPeerConnection(
        onLocalStream: (stream) {
          add(OnLocalStream(mediaStream: stream));
        },
        onAddRemoteStream: (stream) {
          add(OnAddRemoteStream(mediaStream: stream));
        },
        onRemoveRemoteStream: (stream){
          add(OnRemoveRemoteStream(mediaStream: stream));
        }
    );
  }

  _createOffer(RegistrationSession from, RegistrationSession to, bool isCalling, Consultation? consultation) async{
    _oneToOneUseCaseService.createOffer(from: from, to: to, isCalling: isCalling, consultation: consultation);
  }

  _onSocketClose(OnSocketClose event, Emitter<OneToOneUiState> emit) async{
    if(event.from){
      await _oneToOneUseCaseService.close();
    }
    emit(state.copyWith(
        socketState: SocketState(
            socketConnectionState: SocketConnectionState.close,
            exception: event.ex
        )
    ));
  }

  _onConnectToSocket(OnConnectToSocket event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(
      from: event.from,
      to: event.to,
      isCalling: event.isCalling,
      socketState: const SocketState(
          socketConnectionState: SocketConnectionState.connecting
      ),
      mySelf: event.mySelf,
      other: event.other,
      consultationId: event.consultationId
    ));
    try{
      await _oneToOneUseCaseService.connect(url: event.host,
              onOpen: () => add(OnSocketOpen()),
              onMessage: (message) => add(OnSocketMessage(message: message)),
              onClose: (code, reason) => {
                //add(OnSocketClose(ex: reason!, from: false))
              }
      );

    }on Exception catch(e){
      emit(state.copyWith(
        socketState: SocketState(
            socketConnectionState: SocketConnectionState.error,
            exception: e.toString())
      ));
    }
  }

  @override
  Future<void> close() async {
    _oneToOneUseCaseService.close();
    return super.close();
  }


}

==> transmedika_one_to_one_call-1.0.10.tar.gz: lib/webrtc/one_to_one/bloc/one_to_one_bloc.dart <==
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/consultation.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/registration_session.dart';

import '../../../websocket/state/socket_state.dart';
import '../../constants.dart';
import '../models/response/socket_response.dart';
import '../usecase/one_to_one_usecase_service.dart';
import 'one_to_one_event.dart';
import 'one_to_one_state.dart';

class OneToOneBloc extends Bloc<OneToOneUiUIEvent, OneToOneUiState>{
  OneToOneBloc(
      {required OneToOneUseCaseService oneToOneUseCaseService}):
        _oneToOneUseCaseService = oneToOneUseCaseService, super(const OneToOneUiState(
        socketState: SocketState(socketConnectionState: SocketConnectionState.idle)
  )) {
    on<OnConnectToSocket>(
        _onConnectToSocket
    );

    on<OnSocketOpen>(
        _onSocketOpen
    );

    on<OnSocketMessage>(
        _onSocketMessage
    );

    on<OnSocketClose>(
        _onSocketClose
    );

    on<OnRegister>(
        _onRegisterToSocket
    );

    on<OnLocalStream>(
        _onLocalStream
    );

    on<OnAddRemoteStream>(
        _onAddRemoteStream
    );

    on<OnRemoveRemoteStream>(
        _onRemoveRemoteStream
    );

    on<OnCloseSession>(
      _onCloseSession
    );

    on<OnCallAccepted>(
      _onCallAccepted
    );

    on<OnCallRejected>(
      _onCallRejected
    );

    on<OnStopCall>(
      _onStopCall
    );

    on<OnMute>(
      _onMute
    );

    on<OnSwitchCamera>(
        _onSwitchCamera
    );

    on<OnEnableVideo>(
        _onEnableVideo
    );
  }


  final OneToOneUseCaseService _oneToOneUseCaseService;

  _onCloseSession(OnCloseSession event, Emitter<OneToOneUiState> emit) async{
    await _oneToOneUseCaseService.closeSession();
  }

  _onRegisterToSocket(OnRegister event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(callState: CallState.registering));
    await _oneToOneUseCaseService.register(from: state.from!, to: state.to!, isCalling: state.isCalling!);
  }

  _onSocketOpen(OnSocketOpen event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(
        socketState: const SocketState(
            socketConnectionState: SocketConnectionState.open
        )
    ));
    add(OnRegister());
  }

  _onLocalStream(OnLocalStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(localStream: event.mediaStream));
  }

  _onAddRemoteStream(OnAddRemoteStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(remoteStream: event.mediaStream));
  }

  _onRemoveRemoteStream(OnRemoveRemoteStream event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(removeRemoteStream: event.mediaStream));
  }

  _onCallRejected(OnCallRejected event, Emitter<OneToOneUiState> emit) async {
    _oneToOneUseCaseService.reject(from: state.registrationSessionFrom!, to: state.registrationSessionTo!);
    emit(state.copyWith(callState: CallState.callRejectedByUser));
    await _oneToOneUseCaseService.close();
  }

  _onMute(OnMute event, Emitter<OneToOneUiState> emit) async {
    if(state.mute == true){
      emit(state.copyWith(mute: false));
    }else{
      emit(state.copyWith(mute: true));
    }
    _oneToOneUseCaseService.muteMic();
  }

  _onEnableVideo(OnEnableVideo event, Emitter<OneToOneUiState> emit) async {
    if(state.enableVideo == true){
      emit(state.copyWith(enableVideo: false));
    }else{
      emit(state.copyWith(enableVideo: true));
    }
    _oneToOneUseCaseService.enableVideo();
  }

  _onSwitchCamera(OnSwitchCamera event, Emitter<OneToOneUiState> emit) async {
    if(state.frontCamera == true){
      emit(state.copyWith(frontCamera: false));
    }else{
      emit(state.copyWith(frontCamera: true));
    }
    _oneToOneUseCaseService.switchCamera();
  }

  _onStopCall(OnStopCall event, Emitter<OneToOneUiState> emit) async {
    await _oneToOneUseCaseService.closeSession();
    await _oneToOneUseCaseService.close();
  }

  _onCallAccepted(OnCallAccepted event, Emitter<OneToOneUiState> emit) async {
    emit(state.copyWith(callState: CallState.callAcceptedByUser));
    await _createLocalPeerConnection();
    _createOffer(
        state.registrationSessionFrom!,
        state.registrationSessionTo!,
        state.isCalling!,
        Consultation(
            consultationId: state.consultationId!,
            callerUUID: state.mySelf!.uuid,
            calleeUUID:  state.other!.uuid
        )
    );
  }

  _onSocketMessage(OnSocketMessage event, Emitter<OneToOneUiState> emit) async{
    Map<String, dynamic> valueMap = json.decode(event.message);
    SocketResponse response = SocketResponse.fromJson(valueMap);

    emit(state.copyWith(
        socketState: state.socketState?.copyWith(
          message: event.message,
          messageModel: response
        )
    ));

    switch (response.id) {
      case Constants.registerResponse:
        if (response.response == Constants.accepted) {
          emit(state.copyWith(
            callState: CallState.registerAccepted,
            registrationSessionFrom: response.registrationSession
          ));
          _oneToOneUseCaseService.notifyCaller(from: response.registrationSession!, to: state.to!);
        } else {
          emit(state.copyWith(
            callState: CallState.registerRejected,
            registrationSessionFrom: response.registrationSession
          ));
        }
        break;
      case Constants.notifyCallerActive:
        emit(state.copyWith(
            callState: CallState.notifyMe,
            registrationSessionTo: response.from
        ));

        if(state.isCalling == true){
          await _createLocalPeerConnection();
          _createOffer(
            state.registrationSessionFrom!,
            state.registrationSessionTo!,
            state.isCalling!,
            Consultation(
                consultationId: state.consultationId!,
                callerUUID: state.mySelf!.uuid,
                calleeUUID:  state.other!.uuid
            )
          );
        }
        break;
      case Constants.notifyCallerNotActive:
        emit(state.copyWith(
            callState: CallState.missedCall,
            registrationSessionFrom: response.from
        ));
        break;
      case Constants.callResponse:
        if (response.response == Constants.accepted) {
          emit(state.copyWith(callState: CallState.callAccepted));
          _oneToOneUseCaseService.receiveVideoAnswer(response);
        } else {
          emit(state.copyWith(callState: CallState.callRejected));
        }
        break;
      case Constants.incomingCall:
        emit(state.copyWith(
          callState: CallState.incomingCall,
          registrationSessionTo: response.from!
        ));
        break;
      case Constants.startCommunication:
        emit(state.copyWith(callState: CallState.startCommunication));
        _oneToOneUseCaseService.receiveVideoAnswer(response);
        break;
      case Constants.iceCandidate:
        emit(state.copyWith(callState: CallState.iceCandidate));
        _oneToOneUseCaseService.iceCandidate(response);
        break;
      case Constants.stopCommunication:
        emit(state.copyWith(callState: CallState.stopCommunication));
        break;
      default:
        emit(state.copyWith(callState: CallState.unknown));
        debugPrint(Constants.unknown);
    }
  }

  _createLocalPeerConnection() async{
    await _oneToOneUseCaseService.createLocalPeerConnection(
        onLocalStream: (stream) {
          add(OnLocalStream(mediaStream: stream));
        },
        onAddRemoteStream: (stream) {
          add(OnAddRemoteStream(mediaStream: stream));
        },
        onRemoveRemoteStream: (stream){
          add(OnRemoveRemoteStream(mediaStream: stream));
        }
    );
  }

  _createOffer(RegistrationSession from, RegistrationSession to, bool isCalling, Consultation? consultation) async{
    _oneToOneUseCaseService.createOffer(from: from, to: to, isCalling: isCalling, consultation: consultation);
  }

  _onSocketClose(OnSocketClose event, Emitter<OneToOneUiState> emit) async{
    if(event.from){
      await _oneToOneUseCaseService.close();
    }
    emit(state.copyWith(
        socketState: SocketState(
            socketConnectionState: SocketConnectionState.close,
            exception: event.ex
        )
    ));
  }

  _onConnectToSocket(OnConnectToSocket event, Emitter<OneToOneUiState> emit) async{
    emit(state.copyWith(
      from: event.from,
      to: event.to,
      isCalling: event.isCalling,
      socketState: const SocketState(
          socketConnectionState: SocketConnectionState.connecting
      ),
      mySelf: event.mySelf,
      other: event.other,
      consultationId: event.consultationId
    ));
    try{
      await _oneToOneUseCaseService.connect(url: event.host,
              onOpen: () => add(OnSocketOpen()),
              onMessage: (message) => add(OnSocketMessage(message: message)),
              onClose: (code, reason) => {
                //add(OnSocketClose(ex: reason!, from: false))
              }
      );

    }on Exception catch(e){
      emit(state.copyWith(
        socketState: SocketState(
            socketConnectionState: SocketConnectionState.error,
            exception: e.toString())
      ));
    }
  }

  @override
  Future<void> close() async {
    _oneToOneUseCaseService.close();
    return super.close();
  }


}
